# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = ["RequiredBeforeExitToolRule"]


class RequiredBeforeExitToolRule(BaseModel):
    """
    Represents a tool rule configuration where this tool must be called before the agent loop can exit.
    """

    tool_name: str
    """The name of the tool. Must exist in the database for the user's organization."""

    prompt_template: Optional[str] = None
    """Optional template string (ignored)."""

    type: Optional[Literal["required_before_exit"]] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["ConditionalToolRuleParam"]


class ConditionalToolRuleParam(TypedDict, total=False):
    """
    A ToolRule that conditionally maps to different child tools based on the output.
    """

    child_output_mapping: Required[Dict[str, str]]
    """The output case to check for mapping"""

    tool_name: Required[str]
    """The name of the tool. Must exist in the database for the user's organization."""

    default_child: Optional[str]
    """The default child tool to be called. If None, any tool can be called."""

    prompt_template: Optional[str]
    """Optional template string (ignored)."""

    require_output_mapping: bool
    """Whether to throw an error when output doesn't match any case"""

    type: Literal["conditional"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

from .identity_type import IdentityType

__all__ = ["IdentityListParams"]


class IdentityListParams(TypedDict, total=False):
    after: Optional[str]
    """Identity ID cursor for pagination.

    Returns identities that come after this identity ID in the specified sort order
    """

    before: Optional[str]
    """Identity ID cursor for pagination.

    Returns identities that come before this identity ID in the specified sort order
    """

    identifier_key: Optional[str]

    identity_type: Optional[IdentityType]
    """Enum to represent the type of the identity."""

    limit: Optional[int]
    """Maximum number of identities to return"""

    name: Optional[str]

    order: Literal["asc", "desc"]
    """Sort order for identities by creation time.

    'asc' for oldest first, 'desc' for newest first
    """

    order_by: Literal["created_at"]
    """Field to sort by"""

    project_id: Optional[str]
    """[DEPRECATED: Use X-Project-Id header instead] Filter identities by project ID"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Optional
from typing_extensions import TypeAlias

from .passage import Passage
from .._models import BaseModel

__all__ = ["PassageSearchResponse", "PassageSearchResponseItem"]


class PassageSearchResponseItem(BaseModel):
    """Result from a passage search operation with scoring details."""

    passage: Passage
    """The passage object"""

    score: float
    """Relevance score"""

    metadata: Optional[Dict[str, object]] = None
    """Additional metadata about the search result"""


PassageSearchResponse: TypeAlias = List[PassageSearchResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["TemplateDeleteResponse"]


class TemplateDeleteResponse(BaseModel):
    success: bool



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable, Optional
from typing_extensions import Literal, TypedDict

from .._types import SequenceNotStr

__all__ = ["BlockListParams"]


class BlockListParams(TypedDict, total=False):
    after: Optional[str]
    """Block ID cursor for pagination.

    Returns blocks that come after this block ID in the specified sort order
    """

    before: Optional[str]
    """Block ID cursor for pagination.

    Returns blocks that come before this block ID in the specified sort order
    """

    connected_to_agents_count_eq: Optional[Iterable[int]]
    """Filter blocks by the exact number of connected agents.

    If provided, returns blocks that have exactly this number of connected agents.
    """

    connected_to_agents_count_gt: Optional[int]
    """Filter blocks by the number of connected agents.

    If provided, returns blocks that have more than this number of connected agents.
    """

    connected_to_agents_count_lt: Optional[int]
    """Filter blocks by the number of connected agents.

    If provided, returns blocks that have less than this number of connected agents.
    """

    description_search: Optional[str]
    """Search blocks by description.

    If provided, returns blocks whose description matches the search query. This is
    a full-text search on block descriptions.
    """

    identifier_keys: Optional[SequenceNotStr[str]]
    """Search agents by identifier keys"""

    identity_id: Optional[str]
    """The ID of the identity in the format 'identity-<uuid4>'"""

    label: Optional[str]
    """Label to include (alphanumeric, hyphens, underscores only)"""

    label_search: Optional[str]
    """Search blocks by label.

    If provided, returns blocks whose label matches the search query. This is a
    full-text search on block labels.
    """

    limit: Optional[int]
    """Number of blocks to return"""

    name: Optional[str]
    """Name filter (alphanumeric, spaces, hyphens, underscores)"""

    order: Literal["asc", "desc"]
    """Sort order for blocks by creation time.

    'asc' for oldest first, 'desc' for newest first
    """

    order_by: Literal["created_at"]
    """Field to sort by"""

    project_id: Optional[str]
    """Search blocks by project id"""

    templates_only: bool
    """Whether to include only templates"""

    value_search: Optional[str]
    """Search blocks by value.

    If provided, returns blocks whose value matches the search query. This is a
    full-text search on block values.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable, Optional
from typing_extensions import Literal, Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["AgentCreateParams", "InitialMessageSequence"]


class AgentCreateParams(TypedDict, total=False):
    agent_name: str
    """The name of the agent, optional otherwise a random one will be assigned"""

    identity_ids: SequenceNotStr[str]
    """The identity ids to assign to the agent"""

    initial_message_sequence: Iterable[InitialMessageSequence]
    """
    Set an initial sequence of messages, if not provided, the agent will start with
    the default message sequence, if an empty array is provided, the agent will
    start with no messages
    """

    memory_variables: Dict[str, str]
    """The memory variables to assign to the agent"""

    tags: SequenceNotStr[str]
    """The tags to assign to the agent"""

    tool_variables: Dict[str, str]
    """The tool variables to assign to the agent"""


class InitialMessageSequence(TypedDict, total=False):
    content: Required[str]

    role: Required[Literal["user", "system", "assistant"]]

    batch_item_id: Optional[str]

    group_id: Optional[str]

    name: Optional[str]

    otid: Optional[str]

    sender_id: Optional[str]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["ToolCallContentParam"]


class ToolCallContentParam(TypedDict, total=False):
    id: Required[str]
    """A unique identifier for this specific tool call instance."""

    input: Required[Dict[str, object]]
    """
    The parameters being passed to the tool, structured as a dictionary of parameter
    names to values.
    """

    name: Required[str]
    """The name of the tool being called."""

    signature: Optional[str]
    """Stores a unique identifier for any reasoning associated with this tool call."""

    type: Literal["tool_call"]
    """Indicates this content represents a tool call event."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from letta_client import Letta, AsyncLetta
from letta_client.types import ModelListResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestModels:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Letta) -> None:
        model = client.models.list()
        assert_matches_type(ModelListResponse, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Letta) -> None:
        model = client.models.list(
            provider_category=["base", "byok"],
            provider_name="provider_name",
            provider_type="anthropic",
        )
        assert_matches_type(ModelListResponse, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Letta) -> None:
        response = client.models.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = response.parse()
        assert_matches_type(ModelListResponse, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Letta) -> None:
        with client.models.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = response.parse()
            assert_matches_type(ModelListResponse, model, path=["response"])

        assert cast(Any, response.is_closed) is True


class TestAsyncModels:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncLetta) -> None:
        model = await async_client.models.list()
        assert_matches_type(ModelListResponse, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncLetta) -> None:
        model = await async_client.models.list(
            provider_category=["base", "byok"],
            provider_name="provider_name",
            provider_type="anthropic",
        )
        assert_matches_type(ModelListResponse, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncLetta) -> None:
        response = await async_client.models.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        model = await response.parse()
        assert_matches_type(ModelListResponse, model, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncLetta) -> None:
        async with async_client.models.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            model = await response.parse()
            assert_matches_type(ModelListResponse, model, path=["response"])

        assert cast(Any, response.is_closed) is True



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncArrayPage, AsyncArrayPage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.agents import folder_list_params
from ...types.agent_state import AgentState
from ...types.agents.folder_list_response import FolderListResponse

__all__ = ["FoldersResource", "AsyncFoldersResource"]


class FoldersResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> FoldersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return FoldersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FoldersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return FoldersResourceWithStreamingResponse(self)

    def list(
        self,
        agent_id: str,
        *,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        order_by: Literal["created_at"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncArrayPage[FolderListResponse]:
        """
        Get the folders associated with an agent.

        Args:
          agent_id: The ID of the agent in the format 'agent-<uuid4>'

          after: Source ID cursor for pagination. Returns sources that come after this source ID
              in the specified sort order

          before: Source ID cursor for pagination. Returns sources that come before this source ID
              in the specified sort order

          limit: Maximum number of sources to return

          order: Sort order for sources by creation time. 'asc' for oldest first, 'desc' for
              newest first

          order_by: Field to sort by

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return self._get_api_list(
            f"/v1/agents/{agent_id}/folders",
            page=SyncArrayPage[FolderListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "limit": limit,
                        "order": order,
                        "order_by": order_by,
                    },
                    folder_list_params.FolderListParams,
                ),
            ),
            model=FolderListResponse,
        )

    def attach(
        self,
        folder_id: str,
        *,
        agent_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AgentState]:
        """
        Attach a folder to an agent.

        Args:
          agent_id: The ID of the agent in the format 'agent-<uuid4>'

          folder_id: The ID of the source in the format 'source-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not folder_id:
            raise ValueError(f"Expected a non-empty value for `folder_id` but received {folder_id!r}")
        return self._patch(
            f"/v1/agents/{agent_id}/folders/attach/{folder_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AgentState,
        )

    def detach(
        self,
        folder_id: str,
        *,
        agent_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AgentState]:
        """
        Detach a folder from an agent.

        Args:
          agent_id: The ID of the agent in the format 'agent-<uuid4>'

          folder_id: The ID of the source in the format 'source-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not folder_id:
            raise ValueError(f"Expected a non-empty value for `folder_id` but received {folder_id!r}")
        return self._patch(
            f"/v1/agents/{agent_id}/folders/detach/{folder_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AgentState,
        )


class AsyncFoldersResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncFoldersResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFoldersResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFoldersResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return AsyncFoldersResourceWithStreamingResponse(self)

    def list(
        self,
        agent_id: str,
        *,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        order_by: Literal["created_at"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[FolderListResponse, AsyncArrayPage[FolderListResponse]]:
        """
        Get the folders associated with an agent.

        Args:
          agent_id: The ID of the agent in the format 'agent-<uuid4>'

          after: Source ID cursor for pagination. Returns sources that come after this source ID
              in the specified sort order

          before: Source ID cursor for pagination. Returns sources that come before this source ID
              in the specified sort order

          limit: Maximum number of sources to return

          order: Sort order for sources by creation time. 'asc' for oldest first, 'desc' for
              newest first

          order_by: Field to sort by

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return self._get_api_list(
            f"/v1/agents/{agent_id}/folders",
            page=AsyncArrayPage[FolderListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "limit": limit,
                        "order": order,
                        "order_by": order_by,
                    },
                    folder_list_params.FolderListParams,
                ),
            ),
            model=FolderListResponse,
        )

    async def attach(
        self,
        folder_id: str,
        *,
        agent_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AgentState]:
        """
        Attach a folder to an agent.

        Args:
          agent_id: The ID of the agent in the format 'agent-<uuid4>'

          folder_id: The ID of the source in the format 'source-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not folder_id:
            raise ValueError(f"Expected a non-empty value for `folder_id` but received {folder_id!r}")
        return await self._patch(
            f"/v1/agents/{agent_id}/folders/attach/{folder_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AgentState,
        )

    async def detach(
        self,
        folder_id: str,
        *,
        agent_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AgentState]:
        """
        Detach a folder from an agent.

        Args:
          agent_id: The ID of the agent in the format 'agent-<uuid4>'

          folder_id: The ID of the source in the format 'source-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not folder_id:
            raise ValueError(f"Expected a non-empty value for `folder_id` but received {folder_id!r}")
        return await self._patch(
            f"/v1/agents/{agent_id}/folders/detach/{folder_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AgentState,
        )


class FoldersResourceWithRawResponse:
    def __init__(self, folders: FoldersResource) -> None:
        self._folders = folders

        self.list = to_raw_response_wrapper(
            folders.list,
        )
        self.attach = to_raw_response_wrapper(
            folders.attach,
        )
        self.detach = to_raw_response_wrapper(
            folders.detach,
        )


class AsyncFoldersResourceWithRawResponse:
    def __init__(self, folders: AsyncFoldersResource) -> None:
        self._folders = folders

        self.list = async_to_raw_response_wrapper(
            folders.list,
        )
        self.attach = async_to_raw_response_wrapper(
            folders.attach,
        )
        self.detach = async_to_raw_response_wrapper(
            folders.detach,
        )


class FoldersResourceWithStreamingResponse:
    def __init__(self, folders: FoldersResource) -> None:
        self._folders = folders

        self.list = to_streamed_response_wrapper(
            folders.list,
        )
        self.attach = to_streamed_response_wrapper(
            folders.attach,
        )
        self.detach = to_streamed_response_wrapper(
            folders.detach,
        )


class AsyncFoldersResourceWithStreamingResponse:
    def __init__(self, folders: AsyncFoldersResource) -> None:
        self._folders = folders

        self.list = async_to_streamed_response_wrapper(
            folders.list,
        )
        self.attach = async_to_streamed_response_wrapper(
            folders.attach,
        )
        self.detach = async_to_streamed_response_wrapper(
            folders.detach,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from letta_client import Letta, AsyncLetta
from letta_client.types import BlockResponse
from letta_client.pagination import SyncArrayPage, AsyncArrayPage

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestBlocks:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Letta) -> None:
        block = client.blocks.create(
            label="label",
            value="value",
        )
        assert_matches_type(BlockResponse, block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Letta) -> None:
        block = client.blocks.create(
            label="label",
            value="value",
            base_template_id="base_template_id",
            deployment_id="deployment_id",
            description="description",
            entity_id="entity_id",
            hidden=True,
            is_template=True,
            limit=0,
            metadata={"foo": "bar"},
            preserve_on_migration=True,
            project_id="project_id",
            read_only=True,
            template_id="template_id",
            template_name="template_name",
        )
        assert_matches_type(BlockResponse, block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Letta) -> None:
        response = client.blocks.with_raw_response.create(
            label="label",
            value="value",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        block = response.parse()
        assert_matches_type(BlockResponse, block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Letta) -> None:
        with client.blocks.with_streaming_response.create(
            label="label",
            value="value",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            block = response.parse()
            assert_matches_type(BlockResponse, block, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Letta) -> None:
        block = client.blocks.retrieve(
            "block-123e4567-e89b-42d3-8456-426614174000",
        )
        assert_matches_type(BlockResponse, block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Letta) -> None:
        response = client.blocks.with_raw_response.retrieve(
            "block-123e4567-e89b-42d3-8456-426614174000",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        block = response.parse()
        assert_matches_type(BlockResponse, block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Letta) -> None:
        with client.blocks.with_streaming_response.retrieve(
            "block-123e4567-e89b-42d3-8456-426614174000",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            block = response.parse()
            assert_matches_type(BlockResponse, block, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Letta) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `block_id` but received ''"):
            client.blocks.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update(self, client: Letta) -> None:
        block = client.blocks.update(
            block_id="block-123e4567-e89b-42d3-8456-426614174000",
        )
        assert_matches_type(BlockResponse, block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_update_with_all_params(self, client: Letta) -> None:
        block = client.blocks.update(
            block_id="block-123e4567-e89b-42d3-8456-426614174000",
            base_template_id="base_template_id",
            deployment_id="deployment_id",
            description="description",
            entity_id="entity_id",
            hidden=True,
            is_template=True,
            label="label",
            limit=0,
            metadata={"foo": "bar"},
            preserve_on_migration=True,
            project_id="project_id",
            read_only=True,
            template_id="template_id",
            template_name="template_name",
            value="value",
        )
        assert_matches_type(BlockResponse, block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_update(self, client: Letta) -> None:
        response = client.blocks.with_raw_response.update(
            block_id="block-123e4567-e89b-42d3-8456-426614174000",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        block = response.parse()
        assert_matches_type(BlockResponse, block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_update(self, client: Letta) -> None:
        with client.blocks.with_streaming_response.update(
            block_id="block-123e4567-e89b-42d3-8456-426614174000",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            block = response.parse()
            assert_matches_type(BlockResponse, block, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_update(self, client: Letta) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `block_id` but received ''"):
            client.blocks.with_raw_response.update(
                block_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Letta) -> None:
        block = client.blocks.list()
        assert_matches_type(SyncArrayPage[BlockResponse], block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Letta) -> None:
        block = client.blocks.list(
            after="after",
            before="before",
            connected_to_agents_count_eq=[0, 0],
            connected_to_agents_count_gt=0,
            connected_to_agents_count_lt=0,
            description_search="x",
            identifier_keys=["string", "string"],
            identity_id="identity-123e4567-e89b-42d3-8456-426614174000",
            label="human",
            label_search="human",
            limit=0,
            name="My Agent",
            order="asc",
            order_by="created_at",
            project_id="project_id",
            templates_only=True,
            value_search="x",
        )
        assert_matches_type(SyncArrayPage[BlockResponse], block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Letta) -> None:
        response = client.blocks.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        block = response.parse()
        assert_matches_type(SyncArrayPage[BlockResponse], block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Letta) -> None:
        with client.blocks.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            block = response.parse()
            assert_matches_type(SyncArrayPage[BlockResponse], block, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Letta) -> None:
        block = client.blocks.delete(
            "block-123e4567-e89b-42d3-8456-426614174000",
        )
        assert_matches_type(object, block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Letta) -> None:
        response = client.blocks.with_raw_response.delete(
            "block-123e4567-e89b-42d3-8456-426614174000",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        block = response.parse()
        assert_matches_type(object, block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Letta) -> None:
        with client.blocks.with_streaming_response.delete(
            "block-123e4567-e89b-42d3-8456-426614174000",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            block = response.parse()
            assert_matches_type(object, block, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Letta) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `block_id` but received ''"):
            client.blocks.with_raw_response.delete(
                "",
            )


class TestAsyncBlocks:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncLetta) -> None:
        block = await async_client.blocks.create(
            label="label",
            value="value",
        )
        assert_matches_type(BlockResponse, block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncLetta) -> None:
        block = await async_client.blocks.create(
            label="label",
            value="value",
            base_template_id="base_template_id",
            deployment_id="deployment_id",
            description="description",
            entity_id="entity_id",
            hidden=True,
            is_template=True,
            limit=0,
            metadata={"foo": "bar"},
            preserve_on_migration=True,
            project_id="project_id",
            read_only=True,
            template_id="template_id",
            template_name="template_name",
        )
        assert_matches_type(BlockResponse, block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncLetta) -> None:
        response = await async_client.blocks.with_raw_response.create(
            label="label",
            value="value",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        block = await response.parse()
        assert_matches_type(BlockResponse, block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncLetta) -> None:
        async with async_client.blocks.with_streaming_response.create(
            label="label",
            value="value",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            block = await response.parse()
            assert_matches_type(BlockResponse, block, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncLetta) -> None:
        block = await async_client.blocks.retrieve(
            "block-123e4567-e89b-42d3-8456-426614174000",
        )
        assert_matches_type(BlockResponse, block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncLetta) -> None:
        response = await async_client.blocks.with_raw_response.retrieve(
            "block-123e4567-e89b-42d3-8456-426614174000",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        block = await response.parse()
        assert_matches_type(BlockResponse, block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncLetta) -> None:
        async with async_client.blocks.with_streaming_response.retrieve(
            "block-123e4567-e89b-42d3-8456-426614174000",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            block = await response.parse()
            assert_matches_type(BlockResponse, block, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncLetta) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `block_id` but received ''"):
            await async_client.blocks.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update(self, async_client: AsyncLetta) -> None:
        block = await async_client.blocks.update(
            block_id="block-123e4567-e89b-42d3-8456-426614174000",
        )
        assert_matches_type(BlockResponse, block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_update_with_all_params(self, async_client: AsyncLetta) -> None:
        block = await async_client.blocks.update(
            block_id="block-123e4567-e89b-42d3-8456-426614174000",
            base_template_id="base_template_id",
            deployment_id="deployment_id",
            description="description",
            entity_id="entity_id",
            hidden=True,
            is_template=True,
            label="label",
            limit=0,
            metadata={"foo": "bar"},
            preserve_on_migration=True,
            project_id="project_id",
            read_only=True,
            template_id="template_id",
            template_name="template_name",
            value="value",
        )
        assert_matches_type(BlockResponse, block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_update(self, async_client: AsyncLetta) -> None:
        response = await async_client.blocks.with_raw_response.update(
            block_id="block-123e4567-e89b-42d3-8456-426614174000",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        block = await response.parse()
        assert_matches_type(BlockResponse, block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_update(self, async_client: AsyncLetta) -> None:
        async with async_client.blocks.with_streaming_response.update(
            block_id="block-123e4567-e89b-42d3-8456-426614174000",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            block = await response.parse()
            assert_matches_type(BlockResponse, block, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_update(self, async_client: AsyncLetta) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `block_id` but received ''"):
            await async_client.blocks.with_raw_response.update(
                block_id="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncLetta) -> None:
        block = await async_client.blocks.list()
        assert_matches_type(AsyncArrayPage[BlockResponse], block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncLetta) -> None:
        block = await async_client.blocks.list(
            after="after",
            before="before",
            connected_to_agents_count_eq=[0, 0],
            connected_to_agents_count_gt=0,
            connected_to_agents_count_lt=0,
            description_search="x",
            identifier_keys=["string", "string"],
            identity_id="identity-123e4567-e89b-42d3-8456-426614174000",
            label="human",
            label_search="human",
            limit=0,
            name="My Agent",
            order="asc",
            order_by="created_at",
            project_id="project_id",
            templates_only=True,
            value_search="x",
        )
        assert_matches_type(AsyncArrayPage[BlockResponse], block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncLetta) -> None:
        response = await async_client.blocks.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        block = await response.parse()
        assert_matches_type(AsyncArrayPage[BlockResponse], block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncLetta) -> None:
        async with async_client.blocks.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            block = await response.parse()
            assert_matches_type(AsyncArrayPage[BlockResponse], block, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncLetta) -> None:
        block = await async_client.blocks.delete(
            "block-123e4567-e89b-42d3-8456-426614174000",
        )
        assert_matches_type(object, block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncLetta) -> None:
        response = await async_client.blocks.with_raw_response.delete(
            "block-123e4567-e89b-42d3-8456-426614174000",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        block = await response.parse()
        assert_matches_type(object, block, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncLetta) -> None:
        async with async_client.blocks.with_streaming_response.delete(
            "block-123e4567-e89b-42d3-8456-426614174000",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            block = await response.parse()
            assert_matches_type(object, block, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncLetta) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `block_id` but received ''"):
            await async_client.blocks.with_raw_response.delete(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .batches import (
    BatchesResource,
    AsyncBatchesResource,
    BatchesResourceWithRawResponse,
    AsyncBatchesResourceWithRawResponse,
    BatchesResourceWithStreamingResponse,
    AsyncBatchesResourceWithStreamingResponse,
)
from .messages import (
    MessagesResource,
    AsyncMessagesResource,
    MessagesResourceWithRawResponse,
    AsyncMessagesResourceWithRawResponse,
    MessagesResourceWithStreamingResponse,
    AsyncMessagesResourceWithStreamingResponse,
)

__all__ = [
    "MessagesResource",
    "AsyncMessagesResource",
    "MessagesResourceWithRawResponse",
    "AsyncMessagesResourceWithRawResponse",
    "MessagesResourceWithStreamingResponse",
    "AsyncMessagesResourceWithStreamingResponse",
    "BatchesResource",
    "AsyncBatchesResource",
    "BatchesResourceWithRawResponse",
    "AsyncBatchesResourceWithRawResponse",
    "BatchesResourceWithStreamingResponse",
    "AsyncBatchesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncArrayPage, AsyncArrayPage
from ...types.tool import Tool
from ..._base_client import AsyncPaginator, make_request_options
from ...types.agents import tool_run_params, tool_list_params, tool_update_approval_params
from ...types.agent_state import AgentState
from ...types.agents.tool_execution_result import ToolExecutionResult

__all__ = ["ToolsResource", "AsyncToolsResource"]


class ToolsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ToolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return ToolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ToolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return ToolsResourceWithStreamingResponse(self)

    def list(
        self,
        agent_id: str,
        *,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        order_by: Literal["created_at"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncArrayPage[Tool]:
        """
        Get tools from an existing agent.

        Args:
          agent_id: The ID of the agent in the format 'agent-<uuid4>'

          after: Tool ID cursor for pagination. Returns tools that come after this tool ID in the
              specified sort order

          before: Tool ID cursor for pagination. Returns tools that come before this tool ID in
              the specified sort order

          limit: Maximum number of tools to return

          order: Sort order for tools by creation time. 'asc' for oldest first, 'desc' for newest
              first

          order_by: Field to sort by

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return self._get_api_list(
            f"/v1/agents/{agent_id}/tools",
            page=SyncArrayPage[Tool],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "limit": limit,
                        "order": order,
                        "order_by": order_by,
                    },
                    tool_list_params.ToolListParams,
                ),
            ),
            model=Tool,
        )

    def attach(
        self,
        tool_id: str,
        *,
        agent_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AgentState]:
        """
        Attach a tool to an agent.

        Args:
          agent_id: The ID of the agent in the format 'agent-<uuid4>'

          tool_id: The ID of the tool in the format 'tool-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not tool_id:
            raise ValueError(f"Expected a non-empty value for `tool_id` but received {tool_id!r}")
        return self._patch(
            f"/v1/agents/{agent_id}/tools/attach/{tool_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AgentState,
        )

    def detach(
        self,
        tool_id: str,
        *,
        agent_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AgentState]:
        """
        Detach a tool from an agent.

        Args:
          agent_id: The ID of the agent in the format 'agent-<uuid4>'

          tool_id: The ID of the tool in the format 'tool-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not tool_id:
            raise ValueError(f"Expected a non-empty value for `tool_id` but received {tool_id!r}")
        return self._patch(
            f"/v1/agents/{agent_id}/tools/detach/{tool_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AgentState,
        )

    def run(
        self,
        tool_name: str,
        *,
        agent_id: str,
        args: Dict[str, object] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ToolExecutionResult:
        """
        Trigger a tool by name on a specific agent, providing the necessary arguments.

        This endpoint executes a tool that is attached to the agent, using the agent's
        state and environment variables for execution context.

        Args:
          agent_id: The ID of the agent in the format 'agent-<uuid4>'

          args: Arguments to pass to the tool

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not tool_name:
            raise ValueError(f"Expected a non-empty value for `tool_name` but received {tool_name!r}")
        return self._post(
            f"/v1/agents/{agent_id}/tools/{tool_name}/run",
            body=maybe_transform({"args": args}, tool_run_params.ToolRunParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ToolExecutionResult,
        )

    def update_approval(
        self,
        tool_name: str,
        *,
        agent_id: str,
        body_requires_approval: bool,
        query_requires_approval: Optional[bool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AgentState]:
        """
        Modify the approval requirement for a tool attached to an agent.

        Accepts requires_approval via request body (preferred) or query parameter
        (deprecated).

        Args:
          agent_id: The ID of the agent in the format 'agent-<uuid4>'

          body_requires_approval: Whether the tool requires approval before execution

          query_requires_approval: Whether the tool requires approval before execution

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not tool_name:
            raise ValueError(f"Expected a non-empty value for `tool_name` but received {tool_name!r}")
        return self._patch(
            f"/v1/agents/{agent_id}/tools/approval/{tool_name}",
            body=maybe_transform(
                {"body_requires_approval": body_requires_approval}, tool_update_approval_params.ToolUpdateApprovalParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {"query_requires_approval": query_requires_approval},
                    tool_update_approval_params.ToolUpdateApprovalParams,
                ),
            ),
            cast_to=AgentState,
        )


class AsyncToolsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncToolsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return AsyncToolsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncToolsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return AsyncToolsResourceWithStreamingResponse(self)

    def list(
        self,
        agent_id: str,
        *,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        order_by: Literal["created_at"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Tool, AsyncArrayPage[Tool]]:
        """
        Get tools from an existing agent.

        Args:
          agent_id: The ID of the agent in the format 'agent-<uuid4>'

          after: Tool ID cursor for pagination. Returns tools that come after this tool ID in the
              specified sort order

          before: Tool ID cursor for pagination. Returns tools that come before this tool ID in
              the specified sort order

          limit: Maximum number of tools to return

          order: Sort order for tools by creation time. 'asc' for oldest first, 'desc' for newest
              first

          order_by: Field to sort by

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        return self._get_api_list(
            f"/v1/agents/{agent_id}/tools",
            page=AsyncArrayPage[Tool],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "limit": limit,
                        "order": order,
                        "order_by": order_by,
                    },
                    tool_list_params.ToolListParams,
                ),
            ),
            model=Tool,
        )

    async def attach(
        self,
        tool_id: str,
        *,
        agent_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AgentState]:
        """
        Attach a tool to an agent.

        Args:
          agent_id: The ID of the agent in the format 'agent-<uuid4>'

          tool_id: The ID of the tool in the format 'tool-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not tool_id:
            raise ValueError(f"Expected a non-empty value for `tool_id` but received {tool_id!r}")
        return await self._patch(
            f"/v1/agents/{agent_id}/tools/attach/{tool_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AgentState,
        )

    async def detach(
        self,
        tool_id: str,
        *,
        agent_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AgentState]:
        """
        Detach a tool from an agent.

        Args:
          agent_id: The ID of the agent in the format 'agent-<uuid4>'

          tool_id: The ID of the tool in the format 'tool-<uuid4>'

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not tool_id:
            raise ValueError(f"Expected a non-empty value for `tool_id` but received {tool_id!r}")
        return await self._patch(
            f"/v1/agents/{agent_id}/tools/detach/{tool_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AgentState,
        )

    async def run(
        self,
        tool_name: str,
        *,
        agent_id: str,
        args: Dict[str, object] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ToolExecutionResult:
        """
        Trigger a tool by name on a specific agent, providing the necessary arguments.

        This endpoint executes a tool that is attached to the agent, using the agent's
        state and environment variables for execution context.

        Args:
          agent_id: The ID of the agent in the format 'agent-<uuid4>'

          args: Arguments to pass to the tool

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not tool_name:
            raise ValueError(f"Expected a non-empty value for `tool_name` but received {tool_name!r}")
        return await self._post(
            f"/v1/agents/{agent_id}/tools/{tool_name}/run",
            body=await async_maybe_transform({"args": args}, tool_run_params.ToolRunParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ToolExecutionResult,
        )

    async def update_approval(
        self,
        tool_name: str,
        *,
        agent_id: str,
        body_requires_approval: bool,
        query_requires_approval: Optional[bool] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[AgentState]:
        """
        Modify the approval requirement for a tool attached to an agent.

        Accepts requires_approval via request body (preferred) or query parameter
        (deprecated).

        Args:
          agent_id: The ID of the agent in the format 'agent-<uuid4>'

          body_requires_approval: Whether the tool requires approval before execution

          query_requires_approval: Whether the tool requires approval before execution

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not agent_id:
            raise ValueError(f"Expected a non-empty value for `agent_id` but received {agent_id!r}")
        if not tool_name:
            raise ValueError(f"Expected a non-empty value for `tool_name` but received {tool_name!r}")
        return await self._patch(
            f"/v1/agents/{agent_id}/tools/approval/{tool_name}",
            body=await async_maybe_transform(
                {"body_requires_approval": body_requires_approval}, tool_update_approval_params.ToolUpdateApprovalParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {"query_requires_approval": query_requires_approval},
                    tool_update_approval_params.ToolUpdateApprovalParams,
                ),
            ),
            cast_to=AgentState,
        )


class ToolsResourceWithRawResponse:
    def __init__(self, tools: ToolsResource) -> None:
        self._tools = tools

        self.list = to_raw_response_wrapper(
            tools.list,
        )
        self.attach = to_raw_response_wrapper(
            tools.attach,
        )
        self.detach = to_raw_response_wrapper(
            tools.detach,
        )
        self.run = to_raw_response_wrapper(
            tools.run,
        )
        self.update_approval = to_raw_response_wrapper(
            tools.update_approval,
        )


class AsyncToolsResourceWithRawResponse:
    def __init__(self, tools: AsyncToolsResource) -> None:
        self._tools = tools

        self.list = async_to_raw_response_wrapper(
            tools.list,
        )
        self.attach = async_to_raw_response_wrapper(
            tools.attach,
        )
        self.detach = async_to_raw_response_wrapper(
            tools.detach,
        )
        self.run = async_to_raw_response_wrapper(
            tools.run,
        )
        self.update_approval = async_to_raw_response_wrapper(
            tools.update_approval,
        )


class ToolsResourceWithStreamingResponse:
    def __init__(self, tools: ToolsResource) -> None:
        self._tools = tools

        self.list = to_streamed_response_wrapper(
            tools.list,
        )
        self.attach = to_streamed_response_wrapper(
            tools.attach,
        )
        self.detach = to_streamed_response_wrapper(
            tools.detach,
        )
        self.run = to_streamed_response_wrapper(
            tools.run,
        )
        self.update_approval = to_streamed_response_wrapper(
            tools.update_approval,
        )


class AsyncToolsResourceWithStreamingResponse:
    def __init__(self, tools: AsyncToolsResource) -> None:
        self._tools = tools

        self.list = async_to_streamed_response_wrapper(
            tools.list,
        )
        self.attach = async_to_streamed_response_wrapper(
            tools.attach,
        )
        self.detach = async_to_streamed_response_wrapper(
            tools.detach,
        )
        self.run = async_to_streamed_response_wrapper(
            tools.run,
        )
        self.update_approval = async_to_streamed_response_wrapper(
            tools.update_approval,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from typing_extensions import Required, TypeAlias, TypedDict

from .._types import SequenceNotStr
from .dynamic_manager_param import DynamicManagerParam
from .sleeptime_manager_param import SleeptimeManagerParam
from .supervisor_manager_param import SupervisorManagerParam
from .round_robin_manager_param import RoundRobinManagerParam
from .voice_sleeptime_manager_param import VoiceSleeptimeManagerParam

__all__ = ["GroupCreateParams", "ManagerConfig"]


class GroupCreateParams(TypedDict, total=False):
    agent_ids: Required[SequenceNotStr[str]]

    description: Required[str]

    hidden: Optional[bool]
    """If set to True, the group will be hidden."""

    manager_config: ManagerConfig

    project_id: Optional[str]
    """The associated project id."""

    shared_block_ids: SequenceNotStr[str]


ManagerConfig: TypeAlias = Union[
    RoundRobinManagerParam,
    SupervisorManagerParam,
    DynamicManagerParam,
    SleeptimeManagerParam,
    VoiceSleeptimeManagerParam,
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Iterable, Optional
from typing_extensions import Required, TypedDict

from .._types import SequenceNotStr
from .npm_requirement_param import NpmRequirementParam
from .pip_requirement_param import PipRequirementParam

__all__ = ["ToolCreateParams"]


class ToolCreateParams(TypedDict, total=False):
    source_code: Required[str]
    """The source code of the function."""

    args_json_schema: Optional[Dict[str, object]]
    """The args JSON schema of the function."""

    default_requires_approval: Optional[bool]
    """Whether or not to require approval before executing this tool."""

    description: Optional[str]
    """The description of the tool."""

    enable_parallel_execution: Optional[bool]
    """
    If set to True, then this tool will potentially be executed concurrently with
    other tools. Default False.
    """

    json_schema: Optional[Dict[str, object]]
    """
    The JSON schema of the function (auto-generated from source_code if not
    provided)
    """

    npm_requirements: Optional[Iterable[NpmRequirementParam]]
    """Optional list of npm packages required by this tool."""

    pip_requirements: Optional[Iterable[PipRequirementParam]]
    """Optional list of pip packages required by this tool."""

    return_char_limit: int
    """The maximum number of characters in the response."""

    source_type: str
    """The source type of the function."""

    tags: Optional[SequenceNotStr[str]]
    """Metadata tags."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, Annotated, TypeAlias

from .._utils import PropertyInfo
from .._models import BaseModel
from .agents.letta_user_message_content_union import LettaUserMessageContentUnion
from .agents.letta_assistant_message_content_union import LettaAssistantMessageContentUnion

__all__ = [
    "MessageSearchResponse",
    "MessageSearchResponseItem",
    "MessageSearchResponseItemSystemMessageListResult",
    "MessageSearchResponseItemUserMessageListResult",
    "MessageSearchResponseItemReasoningMessageListResult",
    "MessageSearchResponseItemAssistantMessageListResult",
]


class MessageSearchResponseItemSystemMessageListResult(BaseModel):
    """System message list result with agent context.

    Shape is identical to UpdateSystemMessage but includes the owning agent_id.
    """

    content: str
    """
    The message content sent by the system (can be a string or an array of
    multi-modal content parts)
    """

    created_at: datetime
    """The time the message was created in ISO format."""

    agent_id: Optional[str] = None
    """The unique identifier of the agent that owns the message."""

    message_type: Optional[Literal["system_message"]] = None


class MessageSearchResponseItemUserMessageListResult(BaseModel):
    """User message list result with agent context.

    Shape is identical to UpdateUserMessage but includes the owning agent_id.
    """

    content: Union[List[LettaUserMessageContentUnion], str]
    """
    The message content sent by the user (can be a string or an array of multi-modal
    content parts)
    """

    created_at: datetime
    """The time the message was created in ISO format."""

    agent_id: Optional[str] = None
    """The unique identifier of the agent that owns the message."""

    message_type: Optional[Literal["user_message"]] = None


class MessageSearchResponseItemReasoningMessageListResult(BaseModel):
    """Reasoning message list result with agent context.

    Shape is identical to UpdateReasoningMessage but includes the owning agent_id.
    """

    created_at: datetime
    """The time the message was created in ISO format."""

    reasoning: str

    agent_id: Optional[str] = None
    """The unique identifier of the agent that owns the message."""

    message_type: Optional[Literal["reasoning_message"]] = None


class MessageSearchResponseItemAssistantMessageListResult(BaseModel):
    """Assistant message list result with agent context.

    Shape is identical to UpdateAssistantMessage but includes the owning agent_id.
    """

    content: Union[List[LettaAssistantMessageContentUnion], str]
    """
    The message content sent by the assistant (can be a string or an array of
    content parts)
    """

    created_at: datetime
    """The time the message was created in ISO format."""

    agent_id: Optional[str] = None
    """The unique identifier of the agent that owns the message."""

    message_type: Optional[Literal["assistant_message"]] = None


MessageSearchResponseItem: TypeAlias = Annotated[
    Union[
        MessageSearchResponseItemSystemMessageListResult,
        MessageSearchResponseItemUserMessageListResult,
        MessageSearchResponseItemReasoningMessageListResult,
        MessageSearchResponseItemAssistantMessageListResult,
    ],
    PropertyInfo(discriminator="message_type"),
]

MessageSearchResponse: TypeAlias = List[MessageSearchResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable

import httpx

from ..types import access_token_list_params, access_token_create_params, access_token_delete_params
from .._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.access_token_list_response import AccessTokenListResponse
from ..types.access_token_create_response import AccessTokenCreateResponse

__all__ = ["AccessTokensResource", "AsyncAccessTokensResource"]


class AccessTokensResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AccessTokensResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return AccessTokensResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AccessTokensResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return AccessTokensResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        hostname: str,
        policy: Iterable[access_token_create_params.Policy],
        expires_at: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccessTokenCreateResponse:
        """
        Create a new client side access token with the specified configuration.

        Args:
          hostname: The hostname of the client side application. Please specify the full URL
              including the protocol (http or https).

          expires_at: The expiration date of the token. If not provided, the token will expire in 5
              minutes

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/v1/client-side-access-tokens",
            body=maybe_transform(
                {
                    "hostname": hostname,
                    "policy": policy,
                    "expires_at": expires_at,
                },
                access_token_create_params.AccessTokenCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccessTokenCreateResponse,
        )

    def list(
        self,
        *,
        agent_id: str | Omit = omit,
        limit: float | Omit = omit,
        offset: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccessTokenListResponse:
        """List all client side access tokens for the current account.

        This is only
        available for cloud users.

        Args:
          agent_id: The agent ID to filter tokens by. If provided, only tokens for this agent will
              be returned.

          limit: The number of tokens to return per page. Defaults to 10.

          offset: The offset for pagination. Defaults to 0.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._get(
            "/v1/client-side-access-tokens",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "agent_id": agent_id,
                        "limit": limit,
                        "offset": offset,
                    },
                    access_token_list_params.AccessTokenListParams,
                ),
            ),
            cast_to=AccessTokenListResponse,
        )

    def delete(
        self,
        token: str,
        *,
        body: object | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete a client side access token.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not token:
            raise ValueError(f"Expected a non-empty value for `token` but received {token!r}")
        return self._delete(
            f"/v1/client-side-access-tokens/{token}",
            body=maybe_transform(body, access_token_delete_params.AccessTokenDeleteParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AsyncAccessTokensResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAccessTokensResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAccessTokensResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAccessTokensResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return AsyncAccessTokensResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        hostname: str,
        policy: Iterable[access_token_create_params.Policy],
        expires_at: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccessTokenCreateResponse:
        """
        Create a new client side access token with the specified configuration.

        Args:
          hostname: The hostname of the client side application. Please specify the full URL
              including the protocol (http or https).

          expires_at: The expiration date of the token. If not provided, the token will expire in 5
              minutes

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/v1/client-side-access-tokens",
            body=await async_maybe_transform(
                {
                    "hostname": hostname,
                    "policy": policy,
                    "expires_at": expires_at,
                },
                access_token_create_params.AccessTokenCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=AccessTokenCreateResponse,
        )

    async def list(
        self,
        *,
        agent_id: str | Omit = omit,
        limit: float | Omit = omit,
        offset: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AccessTokenListResponse:
        """List all client side access tokens for the current account.

        This is only
        available for cloud users.

        Args:
          agent_id: The agent ID to filter tokens by. If provided, only tokens for this agent will
              be returned.

          limit: The number of tokens to return per page. Defaults to 10.

          offset: The offset for pagination. Defaults to 0.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._get(
            "/v1/client-side-access-tokens",
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "agent_id": agent_id,
                        "limit": limit,
                        "offset": offset,
                    },
                    access_token_list_params.AccessTokenListParams,
                ),
            ),
            cast_to=AccessTokenListResponse,
        )

    async def delete(
        self,
        token: str,
        *,
        body: object | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> object:
        """
        Delete a client side access token.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not token:
            raise ValueError(f"Expected a non-empty value for `token` but received {token!r}")
        return await self._delete(
            f"/v1/client-side-access-tokens/{token}",
            body=await async_maybe_transform(body, access_token_delete_params.AccessTokenDeleteParams),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
        )


class AccessTokensResourceWithRawResponse:
    def __init__(self, access_tokens: AccessTokensResource) -> None:
        self._access_tokens = access_tokens

        self.create = to_raw_response_wrapper(
            access_tokens.create,
        )
        self.list = to_raw_response_wrapper(
            access_tokens.list,
        )
        self.delete = to_raw_response_wrapper(
            access_tokens.delete,
        )


class AsyncAccessTokensResourceWithRawResponse:
    def __init__(self, access_tokens: AsyncAccessTokensResource) -> None:
        self._access_tokens = access_tokens

        self.create = async_to_raw_response_wrapper(
            access_tokens.create,
        )
        self.list = async_to_raw_response_wrapper(
            access_tokens.list,
        )
        self.delete = async_to_raw_response_wrapper(
            access_tokens.delete,
        )


class AccessTokensResourceWithStreamingResponse:
    def __init__(self, access_tokens: AccessTokensResource) -> None:
        self._access_tokens = access_tokens

        self.create = to_streamed_response_wrapper(
            access_tokens.create,
        )
        self.list = to_streamed_response_wrapper(
            access_tokens.list,
        )
        self.delete = to_streamed_response_wrapper(
            access_tokens.delete,
        )


class AsyncAccessTokensResourceWithStreamingResponse:
    def __init__(self, access_tokens: AsyncAccessTokensResource) -> None:
        self._access_tokens = access_tokens

        self.create = async_to_streamed_response_wrapper(
            access_tokens.create,
        )
        self.list = async_to_streamed_response_wrapper(
            access_tokens.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            access_tokens.delete,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .step_list_params import StepListParams as StepListParams
from .message_list_params import MessageListParams as MessageListParams
from .message_stream_params import MessageStreamParams as MessageStreamParams
from .usage_retrieve_response import UsageRetrieveResponse as UsageRetrieveResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from typing_extensions import Literal, TypeAlias, TypedDict

from .tool_return_param import ToolReturnParam
from .approval_return_param import ApprovalReturnParam

__all__ = ["ApprovalCreateParam", "Approval"]

Approval: TypeAlias = Union[ApprovalReturnParam, ToolReturnParam]


class ApprovalCreateParam(TypedDict, total=False):
    """Input to approve or deny a tool call request"""

    approval_request_id: Optional[str]
    """The message ID of the approval request"""

    approvals: Optional[Iterable[Approval]]
    """The list of approval responses"""

    approve: Optional[bool]
    """Whether the tool has been approved"""

    group_id: Optional[str]
    """The multi-agent group that the message was sent in"""

    reason: Optional[str]
    """An optional explanation for the provided approval status"""

    type: Literal["approval"]
    """The message type to be created."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Required, TypeAlias, TypedDict

from .create_sse_mcp_server_param import CreateSseMcpServerParam
from .create_stdio_mcp_server_param import CreateStdioMcpServerParam
from .create_streamable_http_mcp_server_param import CreateStreamableHTTPMcpServerParam

__all__ = ["McpServerCreateParams", "Config"]


class McpServerCreateParams(TypedDict, total=False):
    config: Required[Config]
    """The MCP server configuration (Stdio, SSE, or Streamable HTTP)"""

    server_name: Required[str]
    """The name of the MCP server"""


Config: TypeAlias = Union[CreateStdioMcpServerParam, CreateSseMcpServerParam, CreateStreamableHTTPMcpServerParam]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict
from typing_extensions import Required, TypedDict

__all__ = ["ToolRunParams"]


class ToolRunParams(TypedDict, total=False):
    mcp_server_id: Required[str]

    args: Dict[str, object]
    """Arguments to pass to the tool"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["AgentEnvironmentVariable"]


class AgentEnvironmentVariable(BaseModel):
    agent_id: str
    """The ID of the agent this environment variable belongs to."""

    key: str
    """The name of the environment variable."""

    value: str
    """The value of the environment variable."""

    id: Optional[str] = None
    """The human-friendly ID of the Agent-env"""

    created_at: Optional[datetime] = None
    """The timestamp when the object was created."""

    created_by_id: Optional[str] = None
    """The id of the user that made this object."""

    description: Optional[str] = None
    """An optional description of the environment variable."""

    last_updated_by_id: Optional[str] = None
    """The id of the user that made this object."""

    updated_at: Optional[datetime] = None
    """The timestamp when the object was last updated."""

    value_enc: Optional[str] = None
    """Encrypted secret value (stored as encrypted string)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from tests.utils import assert_matches_type
from letta_client import Letta, AsyncLetta
from letta_client._utils import parse_datetime
from letta_client.types.agents import (
    PassageListResponse,
    PassageCreateResponse,
    PassageSearchResponse,
)

# pyright: reportDeprecated=false

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestPassages:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Letta) -> None:
        with pytest.warns(DeprecationWarning):
            passage = client.agents.passages.create(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
                text="text",
            )

        assert_matches_type(PassageCreateResponse, passage, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Letta) -> None:
        with pytest.warns(DeprecationWarning):
            passage = client.agents.passages.create(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
                text="text",
                created_at=parse_datetime("2019-12-27T18:11:19.117Z"),
                tags=["string"],
            )

        assert_matches_type(PassageCreateResponse, passage, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Letta) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.agents.passages.with_raw_response.create(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
                text="text",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        passage = response.parse()
        assert_matches_type(PassageCreateResponse, passage, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Letta) -> None:
        with pytest.warns(DeprecationWarning):
            with client.agents.passages.with_streaming_response.create(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
                text="text",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                passage = response.parse()
                assert_matches_type(PassageCreateResponse, passage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_create(self, client: Letta) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
                client.agents.passages.with_raw_response.create(
                    agent_id="",
                    text="text",
                )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Letta) -> None:
        with pytest.warns(DeprecationWarning):
            passage = client.agents.passages.list(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
            )

        assert_matches_type(PassageListResponse, passage, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Letta) -> None:
        with pytest.warns(DeprecationWarning):
            passage = client.agents.passages.list(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
                after="after",
                ascending=True,
                before="before",
                limit=0,
                search="search",
            )

        assert_matches_type(PassageListResponse, passage, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Letta) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.agents.passages.with_raw_response.list(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        passage = response.parse()
        assert_matches_type(PassageListResponse, passage, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Letta) -> None:
        with pytest.warns(DeprecationWarning):
            with client.agents.passages.with_streaming_response.list(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                passage = response.parse()
                assert_matches_type(PassageListResponse, passage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_list(self, client: Letta) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
                client.agents.passages.with_raw_response.list(
                    agent_id="",
                )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_delete(self, client: Letta) -> None:
        with pytest.warns(DeprecationWarning):
            passage = client.agents.passages.delete(
                memory_id="memory_id",
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
            )

        assert_matches_type(object, passage, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_delete(self, client: Letta) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.agents.passages.with_raw_response.delete(
                memory_id="memory_id",
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        passage = response.parse()
        assert_matches_type(object, passage, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_delete(self, client: Letta) -> None:
        with pytest.warns(DeprecationWarning):
            with client.agents.passages.with_streaming_response.delete(
                memory_id="memory_id",
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                passage = response.parse()
                assert_matches_type(object, passage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_delete(self, client: Letta) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
                client.agents.passages.with_raw_response.delete(
                    memory_id="memory_id",
                    agent_id="",
                )

            with pytest.raises(ValueError, match=r"Expected a non-empty value for `memory_id` but received ''"):
                client.agents.passages.with_raw_response.delete(
                    memory_id="",
                    agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
                )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_search(self, client: Letta) -> None:
        with pytest.warns(DeprecationWarning):
            passage = client.agents.passages.search(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
                query="query",
            )

        assert_matches_type(PassageSearchResponse, passage, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_search_with_all_params(self, client: Letta) -> None:
        with pytest.warns(DeprecationWarning):
            passage = client.agents.passages.search(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
                query="query",
                end_datetime=parse_datetime("2019-12-27T18:11:19.117Z"),
                start_datetime=parse_datetime("2019-12-27T18:11:19.117Z"),
                tag_match_mode="any",
                tags=["string", "string"],
                top_k=0,
            )

        assert_matches_type(PassageSearchResponse, passage, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_search(self, client: Letta) -> None:
        with pytest.warns(DeprecationWarning):
            response = client.agents.passages.with_raw_response.search(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
                query="query",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        passage = response.parse()
        assert_matches_type(PassageSearchResponse, passage, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_search(self, client: Letta) -> None:
        with pytest.warns(DeprecationWarning):
            with client.agents.passages.with_streaming_response.search(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
                query="query",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                passage = response.parse()
                assert_matches_type(PassageSearchResponse, passage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_search(self, client: Letta) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
                client.agents.passages.with_raw_response.search(
                    agent_id="",
                    query="query",
                )


class TestAsyncPassages:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncLetta) -> None:
        with pytest.warns(DeprecationWarning):
            passage = await async_client.agents.passages.create(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
                text="text",
            )

        assert_matches_type(PassageCreateResponse, passage, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncLetta) -> None:
        with pytest.warns(DeprecationWarning):
            passage = await async_client.agents.passages.create(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
                text="text",
                created_at=parse_datetime("2019-12-27T18:11:19.117Z"),
                tags=["string"],
            )

        assert_matches_type(PassageCreateResponse, passage, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncLetta) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.agents.passages.with_raw_response.create(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
                text="text",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        passage = await response.parse()
        assert_matches_type(PassageCreateResponse, passage, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncLetta) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.agents.passages.with_streaming_response.create(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
                text="text",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                passage = await response.parse()
                assert_matches_type(PassageCreateResponse, passage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_create(self, async_client: AsyncLetta) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
                await async_client.agents.passages.with_raw_response.create(
                    agent_id="",
                    text="text",
                )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncLetta) -> None:
        with pytest.warns(DeprecationWarning):
            passage = await async_client.agents.passages.list(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
            )

        assert_matches_type(PassageListResponse, passage, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncLetta) -> None:
        with pytest.warns(DeprecationWarning):
            passage = await async_client.agents.passages.list(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
                after="after",
                ascending=True,
                before="before",
                limit=0,
                search="search",
            )

        assert_matches_type(PassageListResponse, passage, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncLetta) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.agents.passages.with_raw_response.list(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        passage = await response.parse()
        assert_matches_type(PassageListResponse, passage, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncLetta) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.agents.passages.with_streaming_response.list(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                passage = await response.parse()
                assert_matches_type(PassageListResponse, passage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_list(self, async_client: AsyncLetta) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
                await async_client.agents.passages.with_raw_response.list(
                    agent_id="",
                )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_delete(self, async_client: AsyncLetta) -> None:
        with pytest.warns(DeprecationWarning):
            passage = await async_client.agents.passages.delete(
                memory_id="memory_id",
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
            )

        assert_matches_type(object, passage, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_delete(self, async_client: AsyncLetta) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.agents.passages.with_raw_response.delete(
                memory_id="memory_id",
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        passage = await response.parse()
        assert_matches_type(object, passage, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_delete(self, async_client: AsyncLetta) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.agents.passages.with_streaming_response.delete(
                memory_id="memory_id",
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                passage = await response.parse()
                assert_matches_type(object, passage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_delete(self, async_client: AsyncLetta) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
                await async_client.agents.passages.with_raw_response.delete(
                    memory_id="memory_id",
                    agent_id="",
                )

            with pytest.raises(ValueError, match=r"Expected a non-empty value for `memory_id` but received ''"):
                await async_client.agents.passages.with_raw_response.delete(
                    memory_id="",
                    agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
                )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_search(self, async_client: AsyncLetta) -> None:
        with pytest.warns(DeprecationWarning):
            passage = await async_client.agents.passages.search(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
                query="query",
            )

        assert_matches_type(PassageSearchResponse, passage, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_search_with_all_params(self, async_client: AsyncLetta) -> None:
        with pytest.warns(DeprecationWarning):
            passage = await async_client.agents.passages.search(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
                query="query",
                end_datetime=parse_datetime("2019-12-27T18:11:19.117Z"),
                start_datetime=parse_datetime("2019-12-27T18:11:19.117Z"),
                tag_match_mode="any",
                tags=["string", "string"],
                top_k=0,
            )

        assert_matches_type(PassageSearchResponse, passage, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_search(self, async_client: AsyncLetta) -> None:
        with pytest.warns(DeprecationWarning):
            response = await async_client.agents.passages.with_raw_response.search(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
                query="query",
            )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        passage = await response.parse()
        assert_matches_type(PassageSearchResponse, passage, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_search(self, async_client: AsyncLetta) -> None:
        with pytest.warns(DeprecationWarning):
            async with async_client.agents.passages.with_streaming_response.search(
                agent_id="agent-123e4567-e89b-42d3-8456-426614174000",
                query="query",
            ) as response:
                assert not response.is_closed
                assert response.http_request.headers.get("X-Stainless-Lang") == "python"

                passage = await response.parse()
                assert_matches_type(PassageSearchResponse, passage, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_search(self, async_client: AsyncLetta) -> None:
        with pytest.warns(DeprecationWarning):
            with pytest.raises(ValueError, match=r"Expected a non-empty value for `agent_id` but received ''"):
                await async_client.agents.passages.with_raw_response.search(
                    agent_id="",
                    query="query",
                )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Required, TypedDict

__all__ = ["TemplateUpdateParams"]


class TemplateUpdateParams(TypedDict, total=False):
    agent_file_json: Required[Dict[str, Optional[object]]]
    """The agent file to update the current template version from"""

    save_existing_changes: bool
    """
    If true, Letta will automatically save any changes as a version before updating
    the template
    """

    update_existing_tools: bool
    """
    If true, update existing custom tools source_code and json_schema (source_type
    cannot be changed)
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union
from typing_extensions import Annotated, TypeAlias

from ..._utils import PropertyInfo
from .text_content import TextContent
from .image_content import ImageContent

__all__ = ["LettaUserMessageContentUnion"]

LettaUserMessageContentUnion: TypeAlias = Annotated[
    Union[TextContent, ImageContent], PropertyInfo(discriminator="type")
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Required, Annotated, TypedDict

from ..._types import SequenceNotStr
from ..._utils import PropertyInfo

__all__ = ["PassageCreateParams"]


class PassageCreateParams(TypedDict, total=False):
    text: Required[str]
    """Text to write to archival memory."""

    created_at: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Optional timestamp for the memory (defaults to current UTC time)."""

    tags: Optional[SequenceNotStr[str]]
    """Optional list of tags to attach to the memory."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime

from .._models import BaseModel
from .agents.job_type import JobType
from .stop_reason_type import StopReasonType
from .agents.job_status import JobStatus

__all__ = ["BatchJob"]


class BatchJob(BaseModel):
    id: str
    """The human-friendly ID of the Job"""

    agent_id: Optional[str] = None
    """The agent associated with this job/run."""

    background: Optional[bool] = None
    """Whether the job was created in background mode."""

    callback_error: Optional[str] = None
    """Optional error message from attempting to POST the callback endpoint."""

    callback_sent_at: Optional[datetime] = None
    """Timestamp when the callback was last attempted."""

    callback_status_code: Optional[int] = None
    """HTTP status code returned by the callback endpoint."""

    callback_url: Optional[str] = None
    """If set, POST to this URL when the job completes."""

    completed_at: Optional[datetime] = None
    """The unix timestamp of when the job was completed."""

    created_at: Optional[datetime] = None
    """The unix timestamp of when the job was created."""

    created_by_id: Optional[str] = None
    """The id of the user that made this object."""

    job_type: Optional[JobType] = None

    last_updated_by_id: Optional[str] = None
    """The id of the user that made this object."""

    metadata: Optional[Dict[str, object]] = None
    """The metadata of the job."""

    status: Optional[JobStatus] = None
    """The status of the job."""

    stop_reason: Optional[StopReasonType] = None
    """The reason why the job was stopped."""

    total_duration_ns: Optional[int] = None
    """Total run duration in nanoseconds"""

    ttft_ns: Optional[int] = None
    """Time to first token for a run in nanoseconds"""

    updated_at: Optional[datetime] = None
    """The timestamp when the object was last updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = ["CreateSseMcpServerParam"]


class CreateSseMcpServerParam(TypedDict, total=False):
    """Create a new SSE MCP server"""

    server_url: Required[str]
    """The URL of the server"""

    auth_header: Optional[str]
    """The name of the authentication header (e.g., 'Authorization')"""

    auth_token: Optional[str]
    """The authentication token or API key value"""

    custom_headers: Optional[Dict[str, str]]
    """Custom HTTP headers to include with requests"""

    mcp_server_type: Literal["sse"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .tool import Tool
from .._models import BaseModel

__all__ = ["ToolSearchResult"]


class ToolSearchResult(BaseModel):
    """Result from a tool search operation."""

    combined_score: float
    """Combined relevance score (RRF for hybrid mode)."""

    tool: Tool
    """The matched tool."""

    embedded_text: Optional[str] = None
    """The embedded text content used for matching."""

    fts_rank: Optional[int] = None
    """Full-text search rank position."""

    vector_rank: Optional[int] = None
    """Vector search rank position."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Iterable
from typing_extensions import Required, TypedDict

from ..identity_property_param import IdentityPropertyParam

__all__ = ["PropertyUpsertParams"]


class PropertyUpsertParams(TypedDict, total=False):
    body: Required[Iterable[IdentityPropertyParam]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from ..passage import Passage

__all__ = ["PassageCreateResponse"]

PassageCreateResponse: TypeAlias = List[Passage]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Any, Optional, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._streaming import Stream, AsyncStream
from ...pagination import SyncArrayPage, AsyncArrayPage
from ...types.runs import message_list_params, message_stream_params
from ..._base_client import AsyncPaginator, make_request_options
from ...types.agents.message import Message
from ...types.agents.letta_streaming_response import LettaStreamingResponse

__all__ = ["MessagesResource", "AsyncMessagesResource"]


class MessagesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> MessagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return MessagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> MessagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return MessagesResourceWithStreamingResponse(self)

    def list(
        self,
        run_id: str,
        *,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        order_by: Literal["created_at"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncArrayPage[Message]:
        """
        Get response messages associated with a run.

        Args:
          after: Message ID cursor for pagination. Returns messages that come after this message
              ID in the specified sort order

          before: Message ID cursor for pagination. Returns messages that come before this message
              ID in the specified sort order

          limit: Maximum number of messages to return

          order: Sort order for messages by creation time. 'asc' for oldest first, 'desc' for
              newest first

          order_by: Field to sort by

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._get_api_list(
            f"/v1/runs/{run_id}/messages",
            page=SyncArrayPage[Message],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "limit": limit,
                        "order": order,
                        "order_by": order_by,
                    },
                    message_list_params.MessageListParams,
                ),
            ),
            model=cast(Any, Message),  # Union types cannot be passed in as arguments in the type system
        )

    def stream(
        self,
        run_id: str,
        *,
        batch_size: Optional[int] | Omit = omit,
        include_pings: Optional[bool] | Omit = omit,
        poll_interval: Optional[float] | Omit = omit,
        starting_after: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Stream[LettaStreamingResponse]:
        """
        Retrieve Stream For Run

        Args:
          batch_size: Number of entries to read per batch.

          include_pings: Whether to include periodic keepalive ping messages in the stream to prevent
              connection timeouts.

          poll_interval: Seconds to wait between polls when no new data.

          starting_after: Sequence id to use as a cursor for pagination. Response will start streaming
              after this chunk sequence id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._post(
            f"/v1/runs/{run_id}/stream",
            body=maybe_transform(
                {
                    "batch_size": batch_size,
                    "include_pings": include_pings,
                    "poll_interval": poll_interval,
                    "starting_after": starting_after,
                },
                message_stream_params.MessageStreamParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
            stream=True,
            stream_cls=Stream[LettaStreamingResponse],
        )


class AsyncMessagesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncMessagesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/letta-ai/letta-python#accessing-raw-response-data-eg-headers
        """
        return AsyncMessagesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncMessagesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/letta-ai/letta-python#with_streaming_response
        """
        return AsyncMessagesResourceWithStreamingResponse(self)

    def list(
        self,
        run_id: str,
        *,
        after: Optional[str] | Omit = omit,
        before: Optional[str] | Omit = omit,
        limit: Optional[int] | Omit = omit,
        order: Literal["asc", "desc"] | Omit = omit,
        order_by: Literal["created_at"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[Message, AsyncArrayPage[Message]]:
        """
        Get response messages associated with a run.

        Args:
          after: Message ID cursor for pagination. Returns messages that come after this message
              ID in the specified sort order

          before: Message ID cursor for pagination. Returns messages that come before this message
              ID in the specified sort order

          limit: Maximum number of messages to return

          order: Sort order for messages by creation time. 'asc' for oldest first, 'desc' for
              newest first

          order_by: Field to sort by

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return self._get_api_list(
            f"/v1/runs/{run_id}/messages",
            page=AsyncArrayPage[Message],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "after": after,
                        "before": before,
                        "limit": limit,
                        "order": order,
                        "order_by": order_by,
                    },
                    message_list_params.MessageListParams,
                ),
            ),
            model=cast(Any, Message),  # Union types cannot be passed in as arguments in the type system
        )

    async def stream(
        self,
        run_id: str,
        *,
        batch_size: Optional[int] | Omit = omit,
        include_pings: Optional[bool] | Omit = omit,
        poll_interval: Optional[float] | Omit = omit,
        starting_after: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncStream[LettaStreamingResponse]:
        """
        Retrieve Stream For Run

        Args:
          batch_size: Number of entries to read per batch.

          include_pings: Whether to include periodic keepalive ping messages in the stream to prevent
              connection timeouts.

          poll_interval: Seconds to wait between polls when no new data.

          starting_after: Sequence id to use as a cursor for pagination. Response will start streaming
              after this chunk sequence id

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not run_id:
            raise ValueError(f"Expected a non-empty value for `run_id` but received {run_id!r}")
        return await self._post(
            f"/v1/runs/{run_id}/stream",
            body=await async_maybe_transform(
                {
                    "batch_size": batch_size,
                    "include_pings": include_pings,
                    "poll_interval": poll_interval,
                    "starting_after": starting_after,
                },
                message_stream_params.MessageStreamParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=object,
            stream=True,
            stream_cls=AsyncStream[LettaStreamingResponse],
        )


class MessagesResourceWithRawResponse:
    def __init__(self, messages: MessagesResource) -> None:
        self._messages = messages

        self.list = to_raw_response_wrapper(
            messages.list,
        )
        self.stream = to_raw_response_wrapper(
            messages.stream,
        )


class AsyncMessagesResourceWithRawResponse:
    def __init__(self, messages: AsyncMessagesResource) -> None:
        self._messages = messages

        self.list = async_to_raw_response_wrapper(
            messages.list,
        )
        self.stream = async_to_raw_response_wrapper(
            messages.stream,
        )


class MessagesResourceWithStreamingResponse:
    def __init__(self, messages: MessagesResource) -> None:
        self._messages = messages

        self.list = to_streamed_response_wrapper(
            messages.list,
        )
        self.stream = to_streamed_response_wrapper(
            messages.stream,
        )


class AsyncMessagesResourceWithStreamingResponse:
    def __init__(self, messages: AsyncMessagesResource) -> None:
        self._messages = messages

        self.list = async_to_streamed_response_wrapper(
            messages.list,
        )
        self.stream = async_to_streamed_response_wrapper(
            messages.stream,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Literal, TypedDict

__all__ = ["OmittedReasoningContentParam"]


class OmittedReasoningContentParam(TypedDict, total=False):
    """
    A placeholder for reasoning content we know is present, but isn't returned by the provider (e.g. OpenAI GPT-5 on ChatCompletions)
    """

    signature: Optional[str]
    """A unique identifier for this reasoning step."""

    type: Literal["omitted_reasoning"]
    """Indicates this is an omitted reasoning step."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Any, List, Type, Generic, Mapping, TypeVar, Optional, cast
from typing_extensions import Protocol, override, runtime_checkable

from httpx import Response

from ._utils import is_mapping
from ._models import BaseModel
from ._base_client import BasePage, PageInfo, BaseSyncPage, BaseAsyncPage

__all__ = [
    "SyncArrayPage",
    "AsyncArrayPage",
    "SyncObjectPage",
    "AsyncObjectPage",
    "SyncNextFilesPage",
    "AsyncNextFilesPage",
]

_BaseModelT = TypeVar("_BaseModelT", bound=BaseModel)

_T = TypeVar("_T")


@runtime_checkable
class ArrayPageItem(Protocol):
    id: Optional[str]


@runtime_checkable
class ObjectPageItem(Protocol):
    id: Optional[str]


@runtime_checkable
class NextFilesPageItem(Protocol):
    id: Optional[str]


class SyncArrayPage(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    items: List[_T]

    @override
    def _get_page_items(self) -> List[_T]:
        items = self.items
        if not items:
            return []
        return items

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        is_forwards = not self._options.params.get("before", False)

        items = self.items
        if not items:
            return None

        if is_forwards:
            item = cast(Any, items[-1])
            if not isinstance(item, ArrayPageItem) or item.id is None:
                # TODO emit warning log
                return None

            return PageInfo(params={"after": item.id})
        else:
            item = cast(Any, self.items[0])
            if not isinstance(item, ArrayPageItem) or item.id is None:
                # TODO emit warning log
                return None

            return PageInfo(params={"before": item.id})

    @classmethod
    def build(cls: Type[_BaseModelT], *, response: Response, data: object) -> _BaseModelT:  # noqa: ARG003
        return cls.construct(
            None,
            **{
                **(cast(Mapping[str, Any], data) if is_mapping(data) else {"items": data}),
            },
        )


class AsyncArrayPage(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    items: List[_T]

    @override
    def _get_page_items(self) -> List[_T]:
        items = self.items
        if not items:
            return []
        return items

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        is_forwards = not self._options.params.get("before", False)

        items = self.items
        if not items:
            return None

        if is_forwards:
            item = cast(Any, items[-1])
            if not isinstance(item, ArrayPageItem) or item.id is None:
                # TODO emit warning log
                return None

            return PageInfo(params={"after": item.id})
        else:
            item = cast(Any, self.items[0])
            if not isinstance(item, ArrayPageItem) or item.id is None:
                # TODO emit warning log
                return None

            return PageInfo(params={"before": item.id})

    @classmethod
    def build(cls: Type[_BaseModelT], *, response: Response, data: object) -> _BaseModelT:  # noqa: ARG003
        return cls.construct(
            None,
            **{
                **(cast(Mapping[str, Any], data) if is_mapping(data) else {"items": data}),
            },
        )


class SyncObjectPage(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    messages: List[_T]

    @override
    def _get_page_items(self) -> List[_T]:
        messages = self.messages
        if not messages:
            return []
        return messages

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        is_forwards = not self._options.params.get("before", False)

        messages = self.messages
        if not messages:
            return None

        if is_forwards:
            item = cast(Any, messages[-1])
            if not isinstance(item, ObjectPageItem) or item.id is None:
                # TODO emit warning log
                return None

            return PageInfo(params={"after": item.id})
        else:
            item = cast(Any, self.messages[0])
            if not isinstance(item, ObjectPageItem) or item.id is None:
                # TODO emit warning log
                return None

            return PageInfo(params={"before": item.id})


class AsyncObjectPage(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    messages: List[_T]

    @override
    def _get_page_items(self) -> List[_T]:
        messages = self.messages
        if not messages:
            return []
        return messages

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        is_forwards = not self._options.params.get("before", False)

        messages = self.messages
        if not messages:
            return None

        if is_forwards:
            item = cast(Any, messages[-1])
            if not isinstance(item, ObjectPageItem) or item.id is None:
                # TODO emit warning log
                return None

            return PageInfo(params={"after": item.id})
        else:
            item = cast(Any, self.messages[0])
            if not isinstance(item, ObjectPageItem) or item.id is None:
                # TODO emit warning log
                return None

            return PageInfo(params={"before": item.id})


class SyncNextFilesPage(BaseSyncPage[_T], BasePage[_T], Generic[_T]):
    files: List[_T]
    next_cursor: Optional[str] = None
    has_more: Optional[bool] = None

    @override
    def _get_page_items(self) -> List[_T]:
        files = self.files
        if not files:
            return []
        return files

    @override
    def has_next_page(self) -> bool:
        has_more = self.has_more
        if has_more is not None and has_more is False:
            return False

        return super().has_next_page()

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        is_forwards = not self._options.params.get("before", False)

        files = self.files
        if not files:
            return None

        if is_forwards:
            item = cast(Any, files[-1])
            if not isinstance(item, NextFilesPageItem) or item.id is None:
                # TODO emit warning log
                return None

            return PageInfo(params={"after": item.id})
        else:
            item = cast(Any, self.files[0])
            if not isinstance(item, NextFilesPageItem) or item.id is None:
                # TODO emit warning log
                return None

            return PageInfo(params={"before": item.id})


class AsyncNextFilesPage(BaseAsyncPage[_T], BasePage[_T], Generic[_T]):
    files: List[_T]
    next_cursor: Optional[str] = None
    has_more: Optional[bool] = None

    @override
    def _get_page_items(self) -> List[_T]:
        files = self.files
        if not files:
            return []
        return files

    @override
    def has_next_page(self) -> bool:
        has_more = self.has_more
        if has_more is not None and has_more is False:
            return False

        return super().has_next_page()

    @override
    def next_page_info(self) -> Optional[PageInfo]:
        is_forwards = not self._options.params.get("before", False)

        files = self.files
        if not files:
            return None

        if is_forwards:
            item = cast(Any, files[-1])
            if not isinstance(item, NextFilesPageItem) or item.id is None:
                # TODO emit warning log
                return None

            return PageInfo(params={"after": item.id})
        else:
            item = cast(Any, self.files[0])
            if not isinstance(item, NextFilesPageItem) or item.id is None:
                # TODO emit warning log
                return None

            return PageInfo(params={"before": item.id})



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from typing_extensions import Literal, TypeAlias, TypedDict

from .text_response_format_param import TextResponseFormatParam
from .json_object_response_format_param import JsonObjectResponseFormatParam
from .json_schema_response_format_param import JsonSchemaResponseFormatParam

__all__ = ["AnthropicModelSettingsParam", "ResponseFormat", "Thinking"]

ResponseFormat: TypeAlias = Union[TextResponseFormatParam, JsonSchemaResponseFormatParam, JsonObjectResponseFormatParam]


class Thinking(TypedDict, total=False):
    """The thinking configuration for the model."""

    budget_tokens: int
    """The maximum number of tokens the model can use for extended thinking."""

    type: Literal["enabled", "disabled"]
    """The type of thinking to use."""


class AnthropicModelSettingsParam(TypedDict, total=False):
    effort: Optional[Literal["low", "medium", "high"]]
    """Effort level for Opus 4.5 model (controls token conservation).

    Not setting this gives similar performance to 'high'.
    """

    max_output_tokens: int
    """The maximum number of tokens the model can generate."""

    parallel_tool_calls: bool
    """Whether to enable parallel tool calling."""

    provider_type: Literal["anthropic"]
    """The type of the provider."""

    response_format: Optional[ResponseFormat]
    """The response format for the model."""

    temperature: float
    """The temperature of the model."""

    thinking: Thinking
    """The thinking configuration for the model."""

    verbosity: Optional[Literal["low", "medium", "high"]]
    """Soft control for how verbose model output should be, used for GPT-5 models."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ToolReturnContent"]


class ToolReturnContent(BaseModel):
    content: str
    """The content returned by the tool execution."""

    is_error: bool
    """Indicates whether the tool execution resulted in an error."""

    tool_call_id: str
    """References the ID of the ToolCallContent that initiated this tool call."""

    type: Optional[Literal["tool_return"]] = None
    """Indicates this content represents a tool return event."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, Optional
from datetime import datetime

from .._models import BaseModel

__all__ = ["ProviderTrace"]


class ProviderTrace(BaseModel):
    """Letta's internal representation of a provider trace.

    Attributes:
        id (str): The unique identifier of the provider trace.
        request_json (Dict[str, Any]): JSON content of the provider request.
        response_json (Dict[str, Any]): JSON content of the provider response.
        step_id (str): ID of the step that this trace is associated with.
        organization_id (str): The unique identifier of the organization.
        created_at (datetime): The timestamp when the object was created.
    """

    request_json: Dict[str, object]
    """JSON content of the provider request"""

    response_json: Dict[str, object]
    """JSON content of the provider response"""

    id: Optional[str] = None
    """The human-friendly ID of the Provider_trace"""

    created_at: Optional[datetime] = None
    """The timestamp when the object was created."""

    created_by_id: Optional[str] = None
    """The id of the user that made this object."""

    last_updated_by_id: Optional[str] = None
    """The id of the user that made this object."""

    step_id: Optional[str] = None
    """ID of the step that this trace is associated with"""

    updated_at: Optional[datetime] = None
    """The timestamp when the object was last updated."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from datetime import datetime
from typing_extensions import Literal, Required, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["PassageSearchParams"]


class PassageSearchParams(TypedDict, total=False):
    query: Required[str]
    """Text query for semantic search"""

    agent_id: Optional[str]
    """Filter passages by agent ID"""

    archive_id: Optional[str]
    """Filter passages by archive ID"""

    end_date: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Filter results to passages created before this datetime"""

    limit: int
    """Maximum number of results to return"""

    start_date: Annotated[Union[str, datetime, None], PropertyInfo(format="iso8601")]
    """Filter results to passages created after this datetime"""

    tag_match_mode: Literal["any", "all"]
    """
    How to match tags - 'any' to match passages with any of the tags, 'all' to match
    only passages with all tags
    """

    tags: Optional[SequenceNotStr[str]]
    """Optional list of tags to filter search results"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional
from typing_extensions import Required, TypedDict

from ..._types import SequenceNotStr

__all__ = ["PassageCreateParams"]


class PassageCreateParams(TypedDict, total=False):
    text: Required[str]
    """The text content of the passage"""

    metadata: Optional[Dict[str, object]]
    """Optional metadata for the passage"""

    tags: Optional[SequenceNotStr[str]]
    """Optional tags for categorizing the passage"""
